class BaseEndpoint(object):

	def __init__(self, request, endpoint):
		self.request = request
		self.endpoint = endpoint


class Endpoint(BaseEndpoint):
	def list(self, params = None):
		return self.request.get(self.endpoint, params)

	def create(self, payload):
		return self.request.post(self.endpoint, payload)

	def read(self, id, params = None):
		return self.request.get("{}/{}".format(self.endpoint, id), params)

	def update(self, id, payload):
		return self.request.put("{}/{}".format(self.endpoint, id), payload)

	def delete(self, id):
		return self.request.delete("{}/{}".format(self.endpoint, id))


class Endpoint_Second_Level(BaseEndpoint):
	def list(self, id, params = None):
		return self.request.get("{}/{}/{}".format(self.endpoint['first'], id, self.endpoint['second']), params)

	def create(self, first_id, second_id, payload):
		return self.request.post("{}/{}/{}/{}".format(self.endpoint['first'], first_id, self.endpoint['second'], second_id), payload)

	def read(self, first_id, second_id, params = None):
		return self.request.get("{}/{}/{}/{}".format(self.endpoint['first'], first_id, self.endpoint['second'], second_id), params)

	def update(self, first_id, second_id, payload):
		return self.request.put("{}/{}/{}/{}".format(self.endpoint['first'], first_id, self.endpoint['second'], second_id), payload)

	def delete(self, first_id, second_id):
		return self.request.delete("{}/{}/{}/{}".format(self.endpoint['first'], first_id, self.endpoint['second'], second_id))


class Endpoint_Actions(BaseEndpoint):
	def login(self, payload):
		return self.request.post("{}/{}/login".format(self.endpoint['first'], self.endpoint['second']), payload)

	def activate(self, payload):
		return self.request.post("{}/{}/activate".format(self.endpoint['first'], self.endpoint['second']), payload)

	def resendActivation(self, payload):
		return self.request.post("{}/{}/resend-activation".format(self.endpoint['first'], self.endpoint['second']), payload)

	def forgotPassword(self, payload):
		return self.request.post("{}/{}/forgotten-password".format(self.endpoint['first'], self.endpoint['second']), payload)

	def resetPassword(self, payload):
		return self.request.post("{}/{}/reset-password".format(self.endpoint['first'], self.endpoint['second']), payload)



from .endpoints import Endpoint, Endpoint_Second_Level, Endpoint_Actions
from .request import Request
from . import defaults


class Sugarcoat:
	endpoints = {
		"Store": "stores",
		"Product": "products",
		"Basket": "baskets",
		"ProductTypes": "product-types",
		"User": "users",
		"Customer": "customers",
		"Order": "orders",
		"SearchProducts": "search/products",
		"DiscountCodes": "discount_codes",
		"ProductGroups": "product-groups",
		"Tag": "tags",
		"DeliveryTypes" :"delivery-types",
		"Terms": "terms"
	}

	endpoints_second_level = {
		"BasketProducts": {'first': 'baskets','second': 'products'},
		"BasketDiscount": {'first': 'baskets','second': 'discount_code'},
		"ProductTypeProducts": {'first': 'product-types', 'second': 'products'},
		"OrderComplete": {'first': 'orders', 'second': 'complete'},
		"OrderFail": {'first': 'orders', 'second': 'fail'}
	}

	endpoints_actions = {
		"UserAccount": {'first': 'users','second': 'account'}
	}

	def __init__(self, path=None, key=None):
		if key is None:
			raise RuntimeError("API key must be supplied")

		self.path = path if path else defaults.API_PATH
		self.key = key
		self.request = Request(self.path, self.key)

	def build_endpoint(self, name, base_object, params={}):
		return type(name, (base_object, object), params)

	def __getattr__(self, name):
		if name in self.endpoints:
			obj = self.build_endpoint(name, Endpoint)
			endpoint = obj(self.request, self.endpoints[name])
		elif name in self.endpoints_second_level:
			obj = self.build_endpoint(name, Endpoint_Second_Level, self.endpoints_second_level[name])
			endpoint = obj(self.request, self.endpoints_second_level[name])
		elif name in self.endpoints_actions:
			obj = self.build_endpoint(name, Endpoint_Actions, self.endpoints_actions[name])
			endpoint = obj(self.request, self.endpoints_actions[name])
		else:
			raise RuntimeError("No such wrapper: {}".format(name))

		return endpoint


import requests


class Request:
	def __init__(self, path, key):
		self.base = path
		self.headers = {'Authorization':key, 'content-type': 'application/json'}

	def get(self, endpoint, params):
		return requests.get("{}{}".format(self.base, endpoint), headers=self.headers, params=params)

	def post(self, endpoint, payload):
		return requests.post("{}{}".format(self.base, endpoint), headers=self.headers, json=payload)

	def put(self, endpoint, payload):
		return requests.put("{}{}".format(self.base, endpoint), headers=self.headers, json=payload)

	def delete(self, endpoint):
		return requests.delete("{}{}".format(self.base, endpoint), headers=self.headers)


import unittest
from . import support
from sugarcoat.sugarcoat import Sugarcoat


class TestEndpoints(unittest.TestCase):

	def test_endpoint_fail(self):
		with self.assertRaises(RuntimeError): support.create_endpoint('WrongEndpoint')

	def test_endpoint_pass(self):
		for e in Sugarcoat.endpoints:
			endpoint = support.create_endpoint(e)
			self.assertEqual(Sugarcoat.endpoints[e], endpoint.endpoint)

	def test_basket_products(self):
		endpoint = support.create_endpoint('BasketProducts')
		list = endpoint.list('123')
		self.assertEqual(list.url, "http://localhost/baskets/123/products")

		read = endpoint.read('123', '1')
		self.assertEqual(read.url, "http://localhost/baskets/123/products/1")

		create = endpoint.create('123', '3', {})
		self.assertEqual(create.url, "http://localhost/baskets/123/products/3")

		update = endpoint.update('123', '3', {})
		self.assertEqual(update.url, "http://localhost/baskets/123/products/3")

		delete = endpoint.delete('123', '3')
		self.assertEqual(delete.url, "http://localhost/baskets/123/products/3")

	def test_product_type_products(self):
		endpoint = support.create_endpoint('ProductTypeProducts')
		list = endpoint.list('1')
		self.assertEqual(list.url, "http://localhost/product-types/1/products")

		read = endpoint.read('123', '1')
		self.assertEqual(read.url, "http://localhost/product-types/123/products/1")

		create = endpoint.create('123', '3', {})
		self.assertEqual(create.url, "http://localhost/product-types/123/products/3")

		update = endpoint.update('123', '3', {})
		self.assertEqual(update.url, "http://localhost/product-types/123/products/3")

		delete = endpoint.delete('123', '3')
		self.assertEqual(delete.url, "http://localhost/product-types/123/products/3")

	def test_user_account(self):
		endpoint = support.create_endpoint('UserAccount')
		request = endpoint.login({'username': 'test@test.com', 'password': 'password'})
		self.assertEqual(request.url, "http://localhost/users/account/login")

		request = endpoint.activate({'code': '735tc0d3'})
		self.assertEqual(request.url, "http://localhost/users/account/activate")

		request = endpoint.resendActivation({})
		self.assertEqual(request.url, "http://localhost/users/account/resend-activation")

		request = endpoint.forgotPassword({})
		self.assertEqual(request.url, "http://localhost/users/account/forgotten-password")

		request = endpoint.resetPassword({'code': '735tc0d3'})
		self.assertEqual(request.url, "http://localhost/users/account/reset-password")


class TestWrapper(unittest.TestCase):

	def test_object_instantiator(self):
		sc = Sugarcoat('newpath', '123')
		self.assertEqual(sc.path, 'newpath')
		self.assertEqual(sc.key, '123')


if __name__ == '__main__':
	unittest.main()


import setuptools

with open("README.md", "r") as fh:
	long_description = fh.read()

setuptools.setup(
	name="sugarcoat-sdk",
	version="1.6.1",
	author="Jonathan Butler",
	author_email="jonybutler@gmail.com",
	description="An SDK for the Sugarcoat API",
	long_description=long_description,
	long_description_content_type="text/markdown",
	url="https://gitlab.com/sugarcoat/sugarcoat-python-sdk",
	packages=setuptools.find_packages(),
	classifiers=(
		"Programming Language :: Python :: 3",
		"License :: OSI Approved :: MIT License",
	)
)
